@@ -1,5 +1,14 @@
 import django_filters
 from dcim.models import Device, Site, Region
+from django.conf import settings
+from packaging import version
+
+NETBOX_CURRENT_VERSION = version.parse(settings.VERSION)
+
+if NETBOX_CURRENT_VERSION >= version.parse("2.11.0"):
+    from dcim.models import Location
+else:
+    from dcim.models import RackGroup as Location
 
 
 class TopologyFilterSet(django_filters.FilterSet):
@@ -10,6 +19,10 @@ class TopologyFilterSet(django_filters.FilterSet):
         field_name='id',
         label='Device (ID)',
     )
+    location_id = django_filters.ModelMultipleChoiceFilter(
+        queryset=Location.objects.all(),
+        label='Location (ID)',
+    )
     site_id = django_filters.ModelMultipleChoiceFilter(
         queryset=Site.objects.all(),
         label='Site (ID)',
